@@ -16,7 +16,10 @@ pub async fn create_handler<T: Context + Send, P: Sync + Send + Serialize + Dese
     contact: Option<Contact>,
     shipping_address: Option<Address>,
 ) -> Result<Checkout<P>, Error> {
-    Checkout::new(ctx, currency, contact, shipping_address).await
+    let mut co = Checkout::new(ctx, currency, contact, shipping_address).await?;
+    co.populate_associations(ctx).await?;
+
+    Ok(co)
 }
 
 pub async fn add_item_handler<T: Context + Send, P: Sync + Send + Serialize + DeserializeOwned>(
@@ -35,6 +38,7 @@ pub async fn add_item_handler<T: Context + Send, P: Sync + Send + Serialize + De
     let mut co = maybe_co.unwrap();
 
     co.add_item(ctx, sku, quantity).await?;
+    co.populate_associations(ctx).await?;
 
     Ok(co)
 }
@@ -58,6 +62,7 @@ pub async fn remove_item_handler<
     let mut co = maybe_co.unwrap();
 
     co.remove_item(ctx, sku, quantity).await?;
+    co.populate_associations(ctx).await?;
 
     Ok(co)
 }
@@ -79,6 +84,7 @@ pub async fn confirm_items_handler<
     let mut co = maybe_co.unwrap();
 
     co.confirm_items(ctx).await?;
+    co.populate_associations(ctx).await?;
 
     Ok(co)
 }
@@ -101,6 +107,7 @@ pub async fn update_contact_handler<
     let mut co = maybe_co.unwrap();
 
     co.update_contact(ctx, contact).await?;
+    co.populate_associations(ctx).await?;
 
     Ok(co)
 }
@@ -123,6 +130,7 @@ pub async fn update_shipping_address_handler<
     let mut co = maybe_co.unwrap();
 
     co.update_shipping_address(ctx, shipping_address).await?;
+    co.populate_associations(ctx).await?;
 
     Ok(co)
 }
@@ -145,6 +153,7 @@ pub async fn update_fulfillment_type_handler<
     let mut co = maybe_co.unwrap();
 
     co.update_fulfillment_type(ctx, fulfillment_type).await?;
+    co.populate_associations(ctx).await?;
 
     Ok(co)
 }
@@ -167,6 +176,7 @@ pub async fn initiate_payment_handler<
     let mut co = maybe_co.unwrap();
 
     co.initiate_payment(ctx, args).await?;
+    co.populate_associations(ctx).await?;
 
     Ok(co)
 }
@@ -190,6 +200,7 @@ pub async fn initiate_order_handler<
     let mut co = maybe_co.unwrap();
 
     co.initiate_order(internal_ctx, ctx, args).await?;
+    co.populate_associations(ctx).await?;
 
     Ok(co)
 }
