@@ -29,7 +29,7 @@ pub enum State {
 }
 
 #[derive(Serialize, Deserialize, JsonSchema)]
-pub struct Checkout<P: Sync + Send> {
+pub struct Checkout<P> {
     pub id: String,
     pub state: State,
     pub currency: Currency,
@@ -477,4 +477,15 @@ impl<P: Sync + Send + Serialize + DeserializeOwned> Checkout<P> {
 
         ctx.update_checkout(self).await
     }
+
+    pub async fn populate_associations<C: Context + Send>(
+        &mut self,
+        ctx: &mut C,
+    ) -> Result<(), Error> {
+        if let Some(ref payment_id) = self.payment_id {
+            self.payment = ctx.get_payment(payment_id).await?;
+        }
+
+        Ok(())
+    }
 }
