@@ -1,3 +1,6 @@
+use serde::de::DeserializeOwned;
+use serde::Serialize;
+
 use crate::checkout::Checkout;
 use crate::context::Context;
 use crate::error::new_application_error;
@@ -13,11 +16,11 @@ use super::LookupKey;
 use super::Order;
 use super::OrderState;
 
-pub async fn confirm_handler<C: Context + Send>(
+pub async fn confirm_handler<C: Context + Send, P: Sync + Send + Serialize + DeserializeOwned>(
     internal_ctx: &InternalContext,
     ctx: &mut C,
     order_id: &str,
-) -> Result<Order, Error> {
+) -> Result<Order<P>, Error> {
     let maybe_od = ctx.get_order_for_update(&order_id).await?;
     if maybe_od.is_none() {
         return Err(new_not_found_error(
@@ -31,11 +34,11 @@ pub async fn confirm_handler<C: Context + Send>(
     Ok(od)
 }
 
-pub async fn cancel_handler<C: Context + Send>(
+pub async fn cancel_handler<C: Context + Send, P: Sync + Send + Serialize + DeserializeOwned>(
     internal_ctx: &InternalContext,
     ctx: &mut C,
     order_id: &str,
-) -> Result<Order, Error> {
+) -> Result<Order<P>, Error> {
     // first retrieve the associated payment for update to avoid deadlock
     let maybe_payment = get_order_payment_for_update(ctx, order_id).await?;
 
@@ -60,13 +63,7 @@ pub async fn cancel_handler<C: Context + Send>(
             internal_ctx,
             ctx,
             &mut payment,
-            Initiator::<
-                Payment<
-                    <C as PaymentProcessor>::PrivateData,
-                    <C as PaymentProcessor>::PublicData,
-                    <C as PaymentProcessor>::ProcessArgs,
-                >,
-            >::Order(&mut od),
+            Initiator::<P>::Order(&mut od),
         )
         .await?;
     } else {
@@ -76,11 +73,11 @@ pub async fn cancel_handler<C: Context + Send>(
     Ok(od)
 }
 
-pub async fn reconcile_handler<C: Context + Send>(
+pub async fn reconcile_handler<C: Context + Send, P: Sync + Send + Serialize + DeserializeOwned>(
     ctx: &mut C,
     lookup_key: LookupKey,
 ) -> Result<(), Error> {
-    let maybe_order = match &lookup_key {
+    let maybe_order: Option<Order<P>> = match &lookup_key {
         LookupKey::CheckoutId(checkout_id) => get_order_by_checkout_id(ctx, checkout_id).await?,
         LookupKey::OrderId(order_id) => ctx.get_order_for_update(order_id).await?,
     };
@@ -97,10 +94,13 @@ pub async fn reconcile_handler<C: Context + Send>(
     od.reconcile(ctx).await
 }
 
-async fn get_order_by_checkout_id<C: Context + Send>(
+async fn get_order_by_checkout_id<
+    C: Context + Send,
+    P: Sync + Send + Serialize + DeserializeOwned,
+>(
     ctx: &mut C,
     checkout_id: &str,
-) -> Result<Option<Order>, Error> {
+) -> Result<Option<Order<P>>, Error> {
     let maybe_checkout: Option<
         Checkout<
             Payment<
@@ -139,7 +139,15 @@ async fn get_order_payment_for_update<C: Context + Send>(
     >,
     Error,
 > {
-    let maybe_od = ctx.get_order(order_id).await?;
+    let maybe_od: Option<
+        Order<
+            Payment<
+                <C as PaymentProcessor>::PrivateData,
+                <C as PaymentProcessor>::PublicData,
+                <C as PaymentProcessor>::ProcessArgs,
+            >,
+        >,
+    > = ctx.get_order(order_id).await?;
     if maybe_od.is_none() {
         return Err(new_not_found_error(
             "the requested order could not be found",
