@@ -1,7 +1,7 @@
 use async_trait::async_trait;
 use cloud_pubsub;
 use schemars::JsonSchema;
-use serde::{Deserialize, Serialize};
+use serde::{de::DeserializeOwned, Deserialize, Serialize};
 use serde_json;
 use uuid::Uuid;
 
@@ -13,15 +13,15 @@ use crate::{
     internal_context::InternalContext,
     invoice::Invoice,
     item::Item,
+    payment::PaymentProcessor,
     payment::{handlers as payment_handlers, Initiator},
-    payment::{Payment, PaymentProcessor},
     shipping::FulfillmentType,
 };
 
 pub mod handlers;
 
 #[derive(Serialize, Deserialize, JsonSchema)]
-pub struct Order {
+pub struct Order<P> {
     pub id: String,
     pub state: OrderState,
     pub state_reconciled: bool,
@@ -31,6 +31,8 @@ pub struct Order {
     pub fulfillment_type: FulfillmentType,
     pub invoice: Invoice,
     pub payment_id: Option<String>,
+    // associations
+    pub payment: Option<P>,
 }
 
 #[derive(Serialize, Deserialize, JsonSchema)]
@@ -62,42 +64,69 @@ impl cloud_pubsub::FromPubSubMessage for LookupKey {
 
 #[async_trait]
 pub trait OrderStore {
-    async fn create_order(&mut self, order: &Order) -> Result<(), Error>;
-    async fn update_order(&mut self, order: &Order) -> Result<(), Error>;
-    async fn get_order_for_update(&mut self, id: &str) -> Result<Option<Order>, Error>;
-    async fn get_order(&mut self, id: &str) -> Result<Option<Order>, Error>;
+    async fn create_order<P: Sync + Send + Serialize + DeserializeOwned>(
+        &mut self,
+        order: &Order<P>,
+    ) -> Result<(), Error>;
+    async fn update_order<P: Sync + Send + Serialize + DeserializeOwned>(
+        &mut self,
+        order: &Order<P>,
+    ) -> Result<(), Error>;
+    async fn get_order_for_update<P: Sync + Send + Serialize + DeserializeOwned>(
+        &mut self,
+        id: &str,
+    ) -> Result<Option<Order<P>>, Error>;
+    async fn get_order<P: Sync + Send + Serialize + DeserializeOwned>(
+        &mut self,
+        id: &str,
+    ) -> Result<Option<Order<P>>, Error>;
 }
 
 #[async_trait]
 pub trait OrderEvents {
-    async fn on_payment_in_progress(&mut self, _order: &Order) -> Result<(), Error> {
+    async fn on_payment_in_progress<P: Sync + Send + Serialize + DeserializeOwned>(
+        &mut self,
+        _order: &Order<P>,
+    ) -> Result<(), Error> {
         Ok(())
     }
 
-    async fn on_pending_confirmation(&mut self, _order: &Order) -> Result<(), Error> {
+    async fn on_pending_confirmation<P: Sync + Send + Serialize + DeserializeOwned>(
+        &mut self,
+        _order: &Order<P>,
+    ) -> Result<(), Error> {
         Ok(())
     }
 
-    async fn on_confirmed(&mut self, _order: &Order) -> Result<(), Error> {
+    async fn on_confirmed<P: Sync + Send + Serialize + DeserializeOwned>(
+        &mut self,
+        _order: &Order<P>,
+    ) -> Result<(), Error> {
         Ok(())
     }
 
-    async fn on_fulfilled(&mut self, _order: &Order) -> Result<(), Error> {
+    async fn on_fulfilled<P: Sync + Send + Serialize + DeserializeOwned>(
+        &mut self,
+        _order: &Order<P>,
+    ) -> Result<(), Error> {
         Ok(())
     }
 
-    async fn on_cancelled(&mut self, _order: &Order) -> Result<(), Error> {
+    async fn on_cancelled<P: Sync + Send + Serialize + DeserializeOwned>(
+        &mut self,
+        _order: &Order<P>,
+    ) -> Result<(), Error> {
         Ok(())
     }
 }
 
-impl Order {
-    pub async fn new_from_checkout<C: Context + Send, P: Sync + Send>(
+impl<P: Sync + Send + Serialize + DeserializeOwned> Order<P> {
+    pub async fn new_from_checkout<C: Context + Send>(
         internal_ctx: &InternalContext,
         ctx: &mut C,
         checkout: &Checkout<P>,
     ) -> Result<Self, Error> {
-        let order = Order {
+        let order = Self {
             id: Uuid::new_v4().to_string(),
             state_reconciled: false,
             state: OrderState::PaymentInProgress,
@@ -107,6 +136,8 @@ impl Order {
             fulfillment_type: checkout.fulfillment_type.as_ref().unwrap().clone(),
             invoice: checkout.invoice.as_ref().unwrap().clone(),
             payment_id: checkout.payment_id.clone(),
+            // associations
+            payment: None,
         };
 
         ctx.create_order(&order).await?;
@@ -117,7 +148,7 @@ impl Order {
         Ok(order)
     }
 
-    pub async fn new_from_checkout_missing_payment<C: Context + Send, P: Sync + Send>(
+    pub async fn new_from_checkout_missing_payment<C: Context + Send>(
         internal_ctx: &InternalContext,
         ctx: &mut C,
         checkout: &Checkout<P>,
@@ -133,6 +164,8 @@ impl Order {
             fulfillment_type: checkout.fulfillment_type.as_ref().unwrap().clone(),
             invoice: checkout.invoice.as_ref().unwrap().clone(),
             payment_id: None,
+            // associations
+            payment: None,
         };
 
         ctx.create_order(&order).await?;
@@ -140,13 +173,7 @@ impl Order {
         let charge_amount = order.invoice.initial_charge_amount.clone();
         let payment = payment_handlers::create_handler(
             ctx,
-            Initiator::<
-                Payment<
-                    <C as PaymentProcessor>::PrivateData,
-                    <C as PaymentProcessor>::PublicData,
-                    <C as PaymentProcessor>::ProcessArgs,
-                >,
-            >::Order(&mut order),
+            Initiator::<P>::Order(&mut order),
             charge_amount,
             payment_args,
         )
@@ -163,7 +190,7 @@ impl Order {
     }
 }
 
-impl Order {
+impl<P: Sync + Send + Serialize + DeserializeOwned> Order<P> {
     async fn assert_reconciled(&self) -> Result<(), Error> {
         match self.state_reconciled {
             true => Ok(()),
@@ -252,7 +279,7 @@ impl Order {
     }
 }
 
-impl Order {
+impl<P: Sync + Send + Serialize + DeserializeOwned> Order<P> {
     pub async fn prompt_confirmation<C: Context + Send>(
         &mut self,
         internal_ctx: &InternalContext,
